@@ -19,8 +19,6 @@ Basic overview of Data available in system (suits a PDF/ all is for Web)
 """
 def webReportDataTypesSummary(stationNo, typeInfoById=None):
        
-    ensureWebReportLocations(stationNo)
-
     if typeInfoById == None:
         typeInfoById = expectedTypeCounts(stationNo)
     noRecords = sum(typeInfoById[typId]["count"] for typId in typeInfoById)
@@ -208,10 +206,8 @@ def main():
         stationNo = sys.argv[1]
     except IndexError:
         raise SystemExit("Usage _EXE_ STATIONNO")
-
-    userSiteDir = SITE_DIR_TEMPL.format(stationNo)
-    if not os.path.isdir(userSiteDir):
-        raise SystemExit("Expect User Site to already exist with its basic contents")
+        
+    ensureWebReportLocations(stationNo)
 
     webReportDataTypesSummary(stationNo)
                  
