@@ -103,7 +103,7 @@ class UserClassifier:
             len(self.__userInfoByUserRef)
         ))
         
-        self.__result["postmasterUserRef"] = self.__findPostmaster() # really an enforce 
+        self.__result["postmasterUserRef"] = self.__enforcePostmaster() 
         self.__result["dodUserRef"] = self.__findDoDUser() # one off
                 
         for userRef in self.__userInfoByUserRef:
@@ -146,11 +146,17 @@ class UserClassifier:
     Expect:
     - NO ssn
     """
-    def __findPostmaster(self):
+    def __enforcePostmaster(self):
         postmasters = [userRef for userRef in self.__userInfoByUserRef if re.match(r'POSTMASTER \[', userRef)]
+        if len(postmasters) == 0:
+            raise Exception("No POSTMASTER")
         if len(postmasters) > 1:
-            raise Exception("> 1 user has name POSTMASTER")
-        userRef = postmasters[0]
+            warning = "> 1 User Named Postmaster: {}".format("/".join(postmasters))
+            print("** Warning PostMaster: {}".format(warning))
+        postmaster = [userRef for userRef in postmasters if re.search(r'200\-\.5\]', userRef)]
+        if len(postmaster) != 1:
+            raise Exception("POSTMASTER not 200-.5")
+        userRef = postmaster[0]
         if not re.search(r'\[200-\.5\]$', userRef):
             raise Exception("IEN not .5")
         if userRef not in self.__st3_081sByUserRef:
@@ -391,7 +397,7 @@ class UserClassifier:
         
 
     PUREZEROSOCREATEDUSER_200_MAND_PROPS = set([
-        "name", "alias", "name_components", "signature_block_printed_name", "date_entered", "creator", "xus_logon_attempt_count", "xus_active_user", "secondary_menu_options", "timestamp", "visited_from"
+        "name", "alias", "name_components", "signature_block_printed_name", "date_entered", "xus_logon_attempt_count", "xus_active_user", "secondary_menu_options", "timestamp", "visited_from"
     ])    
     
     PUREZEROSOCREATEDUSER_200_OPT_PROPS = set([    
@@ -464,10 +470,9 @@ class UserClassifier:
             # Do warning on IP Addresses
                 
             return True
-    
-        # Forcing all - key subsetting
-        if "is_created_by_0" not in userInfo:
-            self.__result["remoteExcludeReasonCount"]["0_CREATED"] += 1
+
+        if "creator" in userInfo:
+            self.__result["remoteExcludeReasonCount"]["NO_CREATOR"] += 1
             return False
             
         ALLOWED_PROPS = UserClassifier.PUREZEROSOCREATEDUSER_200_MAND_PROPS.union(UserClassifier.PUREZEROSOCREATEDUSER_200_OPT_PROPS)
