@@ -5,15 +5,17 @@ import sys
 import os
 import re
 import json
-from collections import defaultdict, OrderedDict, Counter
+from collections import defaultdict, Counter
 from datetime import datetime, date
 import numpy
 
 from fmqlutils.reporter.reportUtils import MarkdownTable, reportPercent, reportAbsAndPercent
 
 from fmqlutils.typer.reduceTypeUtils import splitTypeDatas, checkDataPresent, singleValue, combineSubTypes, muBVC, muBVCOfSTProp
-from fmqlreports.webReportUtils import TOP_MD_TEMPL, SITE_DIR_TEMPL, keyStats, flattenFrequencyDistribution, roundFloat, reduce200
+
+from ..webReportUtils import TOP_MD_TEMPL, SITE_DIR_TEMPL, ensureWebReportLocations, keyStats, flattenFrequencyDistribution, roundFloat, reduce200
 from .userClassifier import UserClassifier
+from .USER_CONSTANTS import SSNO_DEFINITIONS
 
 """
 Basic User Type Summary/Overview
@@ -55,7 +57,10 @@ def webReportUser(stationNo, deidentify=False):
     analysis
     """
     print("Loading subtypes ...")
-    type3_081, st3_081URLDs = splitTypeDatas(stationNo, "3_081", reductionLabel="YR1E", expectSubTypeProperties=["user", "level_of_assurance", "remote_app", "is_remote_200_1"])
+    try:
+        type3_081, st3_081URLDs = splitTypeDatas(stationNo, "3_081", reductionLabel="YR1E", expectSubTypeProperties=["user", "level_of_assurance", "remote_app", "remote_200_user_ien", "workstation_label", "workstation_type", "remote_station_id", "device"])
+    except:
+        print("Can't split type data - exiting")
     sts3_081ByUserRef = defaultdict(list)
     for st in st3_081URLDs:
         userRef = singleValue(st, "user")
@@ -95,7 +100,7 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
     if deidentify:
         mu += "__DE-IDENTIFIED__: the names and national identifiers of real (Non-Machine) end users have been scrubbed from this report. VistA-specific IENs are left to identify such users for those with access to the system.\n\n"
             
-    tbl = MarkdownTable(["Type", "Users", "SignOns"], includeNo=False)
+    tbl = MarkdownTable([":Type", "Users", "SignOns"], includeNo=False)
     tbl.addRow([
         "Total Users (200 entries)", 
         len(userInfoByUserRef),
@@ -108,7 +113,7 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
     ])
     tbl.addRow([
         "DoD User",
-        muUserRef(classification["dodUserRef"]),
+        re.search(r'\[200\-([^\]]+)', classification["dodUserRef"]).group(1),
         reportAbsAndPercent(
             st3_081ByUserRef[classification["dodUserRef"]]["_total"],
             type3_081["_total"]
@@ -140,33 +145,21 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
             type3_081["_total"]
         )
     ])
-    tbl.addRow([
-        "Active __Remote Users__", 
-        reportAbsAndPercent(len(classification["activeRemoteUserRefs"]), len(classification["activeUserRefs"])),
-        reportAbsAndPercent(
-            sum(st3_081ByUserRef[userRef]["_total"] for userRef
- in classification["activeRemoteUserRefs"]),
-            type3_081["_total"]
-        )
-    ])
-    tbl.addRow([
-        "Active __Local Users__", 
-        reportAbsAndPercent(len(classification["activeLocalUserRefs"]), len(classification["activeUserRefs"])),
-        reportAbsAndPercent(
-            sum(st3_081ByUserRef[userRef]["_total"] for userRef
- in classification["activeLocalUserRefs"]),
-            type3_081["_total"]
-        )
-    ])
-    tbl.addRow([
-        "Active __Uncategorized__", 
-        reportAbsAndPercent(len(classification["activeNotCategorizedUserRefs"]), len(classification["activeUserRefs"])),
-        reportAbsAndPercent(
-            sum(st3_081ByUserRef[userRef]["_total"] for userRef
- in classification["activeNotCategorizedUserRefs"]),
-            type3_081["_total"]
-        )
-    ])
+    for usrCls, label in [
+        ("activeRemoteUserRefs", "Active __Remote Users__"),
+        ("activeLocalUserRefs", "Active __Local Users__"),
+        ("activeNotCategorizedUserRefs", "Active __Uncategorized__")
+    ]:
+        if len(classification[usrCls]):
+            tbl.addRow([
+                label, 
+                reportAbsAndPercent(len(classification[usrCls]), len(classification["activeUserRefs"])),
+                reportAbsAndPercent(
+                    sum(st3_081ByUserRef[userRef]["_total"] for userRef
+ in classification[usrCls]),
+                    type3_081["_total"]
+                )
+            ])
     
     mu += "User type signon summary ...\n\n"
     mu += tbl.md() + "\n\n"
@@ -198,7 +191,8 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
         classification["warningsByUserRef"],
         userInfoByUserRef,
         st3_081ByUserRef,
-        type3_081["_total"]
+        type3_081["_total"],
+        keyColsOnly=deidentify
     )
 
     mu += webReportNonProxyMachineUsers(
@@ -206,7 +200,8 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
         classification["warningsByUserRef"],
         userInfoByUserRef,
         type3_081["_total"],
-        st3_081ByUserRef
+        st3_081ByUserRef,
+        keyColsOnly = deidentify
     )
 
     mu += webReportRemoteUsers(
@@ -217,7 +212,8 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
         type3_081["_total"], 
         st3_081ByUserRef,
         classification["remoteExcludeReasonCount"], 
-        deidentify
+        deidentify,
+        keyColsOnly = deidentify
     )
         
     # NOTE: far too crude now - need breaks
@@ -228,7 +224,8 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
         userInfoByUserRef, 
         type3_081["_total"], 
         st3_081ByUserRef,
-        deidentify
+        deidentify,
+        keyColsOnly = deidentify
     )
     
     mu += webReportUnclassifiedUsers(
@@ -238,11 +235,12 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
         userInfoByUserRef, 
         type3_081["_total"], 
         st3_081ByUserRef,
-        deidentify
+        deidentify,
+        keyColsOnly = deidentify
     )    
 
     userSiteDir = SITE_DIR_TEMPL.format(stationNo)
-    reportFile = userSiteDir + ("user2.md" if not deidentify else "user2DeId.md")
+    reportFile = userSiteDir + ("user.md" if not deidentify else "userDeId.md")
     print("Writing report {}".format(reportFile))
     open(reportFile, "w").write(mu)
 
@@ -270,14 +268,15 @@ https://github.com/vistadataproject/RPCDefinitionToolkit/issues/44
 Note: tied up with vistalink and two step of connector (with station number
 lookup) and then switch to local user <------------ see if two step means two sign ons?
 """
-def webReportProxyUsers(activeProxyUserRefs, warningsByUserRef, userInfoByUserRef, st3_081ByUserRef, totalSignOns):
+def webReportProxyUsers(activeProxyUserRefs, warningsByUserRef, userInfoByUserRef, st3_081ByUserRef, totalSignOns, keyColsOnly=False):
 
     totalProxySignOnCount = sum(st3_081ByUserRef[userRef]["_total"] for userRef
  in activeProxyUserRefs)
     mu = """## Proxy Users
     
 There are <span class='yellowIt'>{:,}</span> active __Proxy Machine Users__ (user_class is \"CONNECTOR PROXY\") with <span class='yellowIt'>{}</span> signons. All user records have _access_, _verify_ and lack a social while their signons have _LOA_ 2 and don't have \"remote_...\" properties (ie/ CPRS-like combo). These signons probably happen over __VistALink__ and not the plain RPC Broker ...\n\n""".format(len(activeProxyUserRefs), reportAbsAndPercent(totalProxySignOnCount, totalSignOns))
-    tbl = MarkdownTable(["Name [IEN]", "Entered", "PMO", "SMOs", "Keys", "SOs", "Period", "\# IPs", "Duration", "Unexpected"])
+    cols = [":Name [IEN]", "Entered", "SignOns", "Period", "\# IPs"] if keyColsOnly else [":Name [IEN]", "Entered", ":PMO", ":SMOs", ":Keys", "SignOns", "Period", "\# IPs", "Duration", ":Unexpected"] 
+    tbl = MarkdownTable(cols)
     allSTs = []
     for userRef in sorted(activeProxyUserRefs, key=lambda x: st3_081ByUserRef[x]["_total"], reverse=True):
         userInfo = userInfoByUserRef[userRef] 
@@ -295,18 +294,28 @@ There are <span class='yellowIt'>{:,}</span> active __Proxy Machine Users__ (use
         else:
             durMU = ""
         unexpectedMU = "" if userRef not in warningsByUserRef else "/ ".join(warningsByUserRef[userRef])
-        tbl.addRow([
-            muUserRef(userRef),
-            userInfo["date_entered"] if "date_entered" in userInfo else "", 
-            pmoMU,
-            smosMU,
-            keysMU,
-            reportAbsAndPercent(st["_total"], totalProxySignOnCount),
-            muSignOnPeriod(st),
-            muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5),
-            durMU,
-            unexpectedMU
-        ])
+        if keyColsOnly:
+            row = [
+                muUserRef(userRef),
+                userInfo["date_entered"] if "date_entered" in userInfo else "", 
+                reportAbsAndPercent(st["_total"], totalProxySignOnCount),
+                muSignOnPeriod(st),
+                muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5) 
+            ]
+        else:           
+            row = [
+                muUserRef(userRef),
+                userInfo["date_entered"] if "date_entered" in userInfo else "", 
+                pmoMU,
+                smosMU,
+                keysMU,
+                reportAbsAndPercent(st["_total"], totalProxySignOnCount),
+                muSignOnPeriod(st),
+                muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5),
+                durMU,
+                unexpectedMU
+            ]
+        tbl.addRow(row)
     mu += tbl.md() + "\n\n"
             
     return mu
@@ -328,7 +337,7 @@ CVIX_USER_SSNIEN = "200:217122" # expect 2001 too; 2006_95 << sign ons
 ...
 and fixed IP and LOA 1 usually
 """
-def webReportNonProxyMachineUsers(activeNonProxyMachineUserRefs, warningsByUserRef, userInfoByUserRef, totalSignOns, st3_081ByUserRef):
+def webReportNonProxyMachineUsers(activeNonProxyMachineUserRefs, warningsByUserRef, userInfoByUserRef, totalSignOns, st3_081ByUserRef, keyColsOnly=False):
 
     totalNonProxyMachineSignOnCount = sum(st3_081ByUserRef[userRef]["_total"] for userRef in activeNonProxyMachineUserRefs)
     mu = """## (Non Proxy) Machine Users
@@ -339,7 +348,8 @@ Besides the _DoD User_, there are <span class='yellowIt'>{:,}</span> active __No
     )
 
     # To add: workstation_name - take first part? ipv4_address    
-    tbl = MarkdownTable(["Name [IEN]", "Entered", "SSN", "SMOs", "SOs", "Period", "Remote Station Id(s)", "Remote IEN(s)", "IPs", "Duration", "Unexpected"])
+    cols = [":Name [IEN]", "Entered", "SSN", "SignOns", "Period", "Remote Station Id(s)", "Remote IEN(s)", ":IPs"] if keyColsOnly else [":Name [IEN]", "Entered", "SSN", ":SMOs", "SignOns", "Period", "Remote Station Id(s)", "Remote IEN(s)", ":IPs", "Duration", ":Unexpected"]
+    tbl = MarkdownTable(cols)
     for userRef in sorted(activeNonProxyMachineUserRefs, key=lambda x: st3_081ByUserRef[x]["_total"], reverse=True):
         userInfo = userInfoByUserRef[userRef]
         st = st3_081ByUserRef[userRef]
@@ -362,20 +372,34 @@ Besides the _DoD User_, there are <span class='yellowIt'>{:,}</span> active __No
                 durMU = muSeconds(singleValue(st, "duration"))
         else:
             durMU = ""
-        tbl.addRow([
-            muUserRef(userRef),
-            userInfo["date_entered"] if "date_entered" in userInfo else "",
-            "__{}__".format(userInfo["ssn"]) if "ssn" in userInfo else "",
-            smosMU,
-            reportAbsAndPercent(st["_total"], totalNonProxyMachineSignOnCount),
-            muSignOnPeriod(st),
-            # NO remote app?
-            "/".join(st["remote_station_id"]["byValueCount"]) if "remote_station_id" in st else "",
-            remoteIENsMU,
-            muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5),
-            durMU,
-            unexpectedMU      
-        ])
+        if keyColsOnly:
+            row = [
+                muUserRef(userRef),
+                userInfo["date_entered"] if "date_entered" in userInfo else "",
+                "__{}__".format(userInfo["ssn"]) if "ssn" in userInfo else "",
+                reportAbsAndPercent(st["_total"], totalNonProxyMachineSignOnCount),
+                muSignOnPeriod(st),
+                # NO remote app?
+                "/".join(st["remote_station_id"]["byValueCount"].keys()) if "remote_station_id" in st else "",
+                remoteIENsMU,
+                muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5)            
+            ]
+        else:
+            row = [
+                muUserRef(userRef),
+                userInfo["date_entered"] if "date_entered" in userInfo else "",
+                "__{}__".format(userInfo["ssn"]) if "ssn" in userInfo else "",
+                smosMU,
+                reportAbsAndPercent(st["_total"], totalNonProxyMachineSignOnCount),
+                muSignOnPeriod(st),
+                # NO remote app?
+                "/".join(st["remote_station_id"]["byValueCount"].keys()) if "remote_station_id" in st else "",
+                remoteIENsMU,
+                muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5),
+                durMU,
+                unexpectedMU      
+            ]
+        tbl.addRow(row)
     mu += tbl.md() + "\n\n"
         
     return mu
@@ -399,9 +423,9 @@ def webReportDoDUser(userRef, userInfo, totalSignons, st, warnings, deidentify):
     
 One special non proxy, machine user, the __\"DoD User\"__ is used for JLV DoD access and for access by a number of other applications ...\n\n"""
     
-    tbl = MarkdownTable(["Property", "Value"])
+    tbl = MarkdownTable([":Property", ":Value"], includeNo=False)
     
-    tbl.addRow(["Name \[IEN\]", muUserRef(userRef)])
+    tbl.addRow(["IEN", re.search(r'\[200\-([^\]]+)', userRef).group(1)])
     tbl.addRow(["Date Entered", userInfo["date_entered"] if "date_entered" in userInfo else ""])
     tbl.addRow(["SSN", "__{}__".format(userInfo["ssn"])])
     pmoMU, smosMU, keysMU = muOptionsNKeys(userInfo)
@@ -413,7 +437,7 @@ One special non proxy, machine user, the __\"DoD User\"__ is used for JLV DoD ac
     noRemoteStationIds = len(st["remote_station_id"]["byValueCount"])
     tbl.addRow(["Station Ids", noRemoteStationIds])
     def topRemoteStationIds(st):
-        orderedTopCounts = OrderedDict()
+        orderedTopCounts = {} 
         for i, rsid in enumerate(sorted(st["remote_station_id"]["byValueCount"], key=lambda x: st["remote_station_id"]["byValueCount"][x], reverse=True), 1):
             if i > 5:
                 break
@@ -440,7 +464,7 @@ Every VistA has __Postmaster__, (one of) the first user in the system ...
 
 """
     
-    tbl = MarkdownTable(["Property", "Value"])
+    tbl = MarkdownTable([":Property", ":Value"], includeNo=False)
     
     tbl.addRow(["Name \[IEN\]", muUserRef(userRef)])
     tbl.addRow(["Date Entered", userInfo["date_entered"] if "date_entered" in userInfo else ""])
@@ -486,11 +510,14 @@ TODO: (review the issues in TK too)
 - manilla called out; 200 too?
 fmrepo/user/catag (move over)
 """
-def webReportRemoteUsers(activeRemoteUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, remoteExcludeReasonCount, deidentify):
+def webReportRemoteUsers(activeRemoteUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, remoteExcludeReasonCount, deidentify, keyColsOnly=False):
+
+    if len(activeRemoteUserRefs) == 0:
+        return ""
 
     remoteSignOnCountsByUserRef = dict((userRef, st3_081ByUserRef[userRef]["_total"]) for userRef in activeRemoteUserRefs)
     totalRemoteSignOns = sum(remoteSignOnCountsByUserRef[userRef] for userRef in remoteSignOnCountsByUserRef)
-    kstats = keyStats(remoteSignOnCountsByUserRef.values())
+    kstats = keyStats(list(remoteSignOnCountsByUserRef.values()))
 
     print("About to combine lot's of remote sts (takes time) ...")
     comboST = combineSubTypes([st3_081ByUserRef[userRef] for userRef in activeRemoteUserRefs], forceCountProps=["remote_station_id", "remote_app", "division"])[0]
@@ -511,13 +538,13 @@ Remote users dominate in every VistA - <span class='yellowIt'>{}</span> - but ac
         len(remoteStationBVC),
         len(set(stId[0:3] for stId in remoteStationBVC))
     )    
-    tbl = MarkdownTable(["Remote Station Id", "Count of Remote SignOns"])
+    tbl = MarkdownTable([":Remote Station Id", ":Count of Remote SignOns"], includeNo=False)
     for i, stationId in enumerate(sorted(remoteStationBVC, key=lambda x: remoteStationBVC[x], reverse=True), 1):
         if i > 10:
             break
         stationMU = "__{}__".format(stationId)
-        if stationId == "459": # Tripler
-            stationMU = "__459 [Tripler/JLV Test]__"
+        if stationId in SSNO_DEFINITIONS:
+            stationMU = "__{} [{}]__".format(stationId, SSNO_DEFINITIONS[stationId])
         tbl.addRow([
             stationMU, 
             reportAbsAndPercent(remoteStationBVC[stationId], totalRemoteSignOns)
@@ -550,11 +577,7 @@ Remote users dominate in every VistA - <span class='yellowIt'>{}</span> - but ac
         # TODO: match counts in both to assemble id
         return ""
     mu += "The following shows the top 50 Remote Users ...\n\n"
-    cols = ["Name [IEN]", "Entry", "Remote Id(s)", "SOs", "Period", "Days", "Remote Apps", "Options", "Duration"]
-    useUnexpected = False
-    if sum(1 for userRef in activeRemoteUserRefs if userRef in warningsByUserRef):
-        useUnexpected = True
-        cols.append("Unexpected") 
+    cols = [":Name [IEN]", "Entry", "Remote Id(s)", "SignOns", "Period", "Days", ":Remote Apps"] if keyColsOnly else [":Name [IEN]", "Entry", "Remote Id(s)", "SignOns", "Period", "Days", ":Remote Apps", ":Options", "Duration", ":Unexpected"]   
     tbl = MarkdownTable(cols)
     for i, userRef in enumerate(sorted(activeRemoteUserRefs, key=lambda x: st3_081ByUserRef[x]["_total"], reverse=True), 1):
         if i > 50:
@@ -579,41 +602,44 @@ Remote users dominate in every VistA - <span class='yellowIt'>{}</span> - but ac
                 durMU = muSeconds(singleValue(st, "duration"))
         else:
             durMU = ""
-        row = [
-            muUserRef(userRef, ssn="NO SSN" if "ssn" not in userInfo else userInfo["ssn"], deidentify=deidentify),
-            userInfo["date_entered"] if "date_entered" in userInfo else "",
-            muRemoteIds(st),
-            reportAbsAndPercent(st["_total"], totalRemoteSignOns),
-            muSignOnPeriod(st),
+        if keyColsOnly:
+            row = [ 
+                muUserRef(userRef, ssn="NO SSN" if "ssn" not in userInfo else userInfo["ssn"], deidentify=deidentify),
+                userInfo["date_entered"] if "date_entered" in userInfo else "",
+                muRemoteIds(st),
+                reportAbsAndPercent(st["_total"], totalRemoteSignOns),
+                muSignOnPeriod(st),
             ", ".join(["{} [{}]".format("__{}__".format(day) if i < 5 else day, wdCntr[day]) for i, day in enumerate(wdCntr)]),
-            remoteAppMU,
-            smosMU,
-            durMU
-        ]
-        if useUnexpected:
-            unexpectedMU = "" if userRef not in warningsByUserRef else "/ ".join(warningsByUserRef[userRef])
-            row.append(unexpectedMU)
+                remoteAppMU            
+            ]
+        else:
+            row = [
+                muUserRef(userRef, ssn="NO SSN" if "ssn" not in userInfo else userInfo["ssn"], deidentify=deidentify),
+                userInfo["date_entered"] if "date_entered" in userInfo else "",
+                muRemoteIds(st),
+                reportAbsAndPercent(st["_total"], totalRemoteSignOns),
+                muSignOnPeriod(st),
+            ", ".join(["{} [{}]".format("__{}__".format(day) if i < 5 else day, wdCntr[day]) for i, day in enumerate(wdCntr)]),
+                remoteAppMU,
+                smosMU,
+                durMU,
+                "" if userRef not in warningsByUserRef else "/ ".join(warningsByUserRef[userRef])
+            ]
         tbl.addRow(row)
     mu += tbl.md() + "\n\n"
         
     return mu
     
-def webReportUnclassifiedUsers(activeNotCategorizedUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, deidentify):
-
-    totalUnclassfiedSignOns = sum(st3_081ByUserRef[userRef]["_total"] for userRef in activeNotCategorizedUserRefs)
-    mu = """## Unclassified Users
-    
-There are <span class='yellowIt'>{}</span> active unclassified Users with <span class='yellowIt'>{}</span> signons.\n\n""".format(reportAbsAndPercent(len(activeNotCategorizedUserRefs), totalActiveUsers), reportAbsAndPercent(totalUnclassfiedSignOns, totalSignOns))
-
-    return mu
-    
 """
 TODO: 
 - broad: FAR TOO catch all -- why does PUG have so many more in Pug than ALake? 
   - and device: ssh vs other
 - exclude from remote on MAND_PROPS and ALLOWED_PROPS as opposed to 0's ie/ 0's in here PLUS station_no in locals
 """
-def webReportLocalUsers(activeLocalUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, deidentify):
+def webReportLocalUsers(activeLocalUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, deidentify, keyColsOnly=False):
+
+    if len(activeLocalUserRefs) == 0:
+        return ""
 
     totalLocalSignOns = sum(st3_081ByUserRef[userRef]["_total"] for userRef in activeLocalUserRefs)
     mu = """## Local Users
@@ -622,7 +648,7 @@ There are <span class='yellowIt'>{}</span> active Local Users with <span class='
 
     comboST = combineSubTypes([st3_081ByUserRef[userRef] for userRef in activeLocalUserRefs], forceCountProps=["division"])[0]
     mu += "Local users select <span class='yellowIt'>{:,}</span> divisions (SHOULD RESTRICT THIS SET) ...\n\n".format(len(comboST["division"]["byValueCount"]))
-    tbl = MarkdownTable(["Division", "Count %"])
+    tbl = MarkdownTable([":Division", ":Count %"])
     for divisionRef in sorted(comboST["division"]["byValueCount"], key=lambda x: comboST["division"]["byValueCount"][x], reverse=True):
         tbl.addRow([
             "__{}__".format(re.sub(r'4\-', '', divisionRef)), 
@@ -631,7 +657,7 @@ There are <span class='yellowIt'>{}</span> active Local Users with <span class='
     mu += tbl.md() + "\n\n"
     
     mu += "And don't just sign on through device 0 ...\n\n"
-    tbl = MarkdownTable(["Device", "Count %"])
+    tbl = MarkdownTable([":Device", "Count %"], includeNo=False)
     for device in sorted(comboST["device"]["byValueCount"], key=lambda x: comboST["device"]["byValueCount"][x], reverse=True):
         tbl.addRow([
             "__{}__".format(device), 
@@ -641,7 +667,7 @@ There are <span class='yellowIt'>{}</span> active Local Users with <span class='
     
     # TODO: will break on these 
     mu += "And have multiple levels of assurance ...\n\n"
-    tbl = MarkdownTable(["LOA", "Count %"])
+    tbl = MarkdownTable([":LOA", "Count %"], includeNo=False)
     for loa in sorted(comboST["level_of_assurance"]["byValueCount"], key=lambda x: comboST["level_of_assurance"]["byValueCount"][x], reverse=True):
         tbl.addRow([
             "__{}__".format(device), 
@@ -652,37 +678,44 @@ There are <span class='yellowIt'>{}</span> active Local Users with <span class='
     SUPERUSER_KEYS = ["XUMGR"] # removing XUPROG
     superUserRefs = set(userRef for userRef in activeLocalUserRefs if "keys" in userInfoByUserRef[userRef] and len(set(SUPERUSER_KEYS).intersection(set(userInfoByUserRef[userRef]["keys"]))))
     mu += "<span class='yellowIt'>{:,}</span> Local Users are __Superusers__ (those with key {}) ...\n\n".format(len(superUserRefs), "|".join(SUPERUSER_KEYS))
-    cols = ["Name [IEN]", "Entry", "Title", "SOs", "Period", "Days", "PMO", "SMOs", "Keys"]
-    useUnexpected = False
-    if sum(1 for userRef in superUserRefs if userRef in warningsByUserRef):
-        useUnexpected = True
-        cols.append("Unexpected") 
+    cols = [":Name [IEN]", "Entry", ":Title", "SignOns", "Period", "Days"] if keyColsOnly else [":Name [IEN]", "Entry", ":Title", "SignOns", "Period", "Days", ":PMO", ":SMOs", ":Keys", ":Unexpected"]
     tbl = MarkdownTable(cols)
     for userRef in sorted(superUserRefs, key=lambda x: st3_081ByUserRef[x]["_total"], reverse=True):
         userInfo = userInfoByUserRef[userRef]
         st = st3_081ByUserRef[userRef]
         pmoMU, smosMU, keysMU = muOptionsNKeys(userInfo)
         wdCntr = expandByWeekDay(st)
-        row = [
-            muUserRef(userRef, ssn="NO SSN" if "ssn" not in userInfo else userInfo["ssn"], deidentify=deidentify),
-            userInfo["date_entered"] if "date_entered" in userInfo else "",
-            userInfo["title"] if "title" in userInfo else "",
-            reportAbsAndPercent(st["_total"], totalLocalSignOns),
-            muSignOnPeriod(st),
-            ", ".join(["{} [{}]".format("__{}__".format(day) if i < 5 else day, wdCntr[day]) for i, day in enumerate(wdCntr)]),
-            pmoMU,
-            smosMU,
-            keysMU
-        ]
-        if useUnexpected:
-            unexpectedMU = "" if userRef not in warningsByUserRef else "/ ".join(warningsByUserRef[userRef])
-            row.append(unexpectedMU)
+        if keyColsOnly:
+            row = [
+                muUserRef(userRef, ssn="NO SSN" if "ssn" not in userInfo else userInfo["ssn"], deidentify=deidentify),
+                userInfo["date_entered"] if "date_entered" in userInfo else "",
+                userInfo["title"] if "title" in userInfo else "",
+                reportAbsAndPercent(st["_total"], totalLocalSignOns),
+                muSignOnPeriod(st),
+                ", ".join(["{} [{}]".format("__{}__".format(day) if i < 5 else day, wdCntr[day]) for i, day in enumerate(wdCntr)])            
+            ]
+        else:
+            row = [
+                muUserRef(userRef, ssn="NO SSN" if "ssn" not in userInfo else userInfo["ssn"], deidentify=deidentify),
+                userInfo["date_entered"] if "date_entered" in userInfo else "",
+                userInfo["title"] if "title" in userInfo else "",
+                reportAbsAndPercent(st["_total"], totalLocalSignOns),
+                muSignOnPeriod(st),
+                ", ".join(["{} [{}]".format("__{}__".format(day) if i < 5 else day, wdCntr[day]) for i, day in enumerate(wdCntr)]),
+                pmoMU,
+                smosMU,
+                keysMU,
+                "" if userRef not in warningsByUserRef else "/ ".join(warningsByUserRef[userRef])
+            ]
         tbl.addRow(row)
     mu += tbl.md() + "\n\n"
         
     return mu
     
-def webReportUnclassifiedUsers(activeNotCategorizedUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, deidentify):
+def webReportUnclassifiedUsers(activeNotCategorizedUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, deidentify, keyColsOnly=False):
+
+    if len(activeNotCategorizedUserRefs) == 0:
+        return ""
 
     totalUnclassfiedSignOns = sum(st3_081ByUserRef[userRef]["_total"] for userRef in activeNotCategorizedUserRefs)
     mu = """## Unclassified Users
@@ -742,7 +775,7 @@ def muUserRef(userRef, ssn=None, deidentify=False):
     
 def tblByWeekDay(st):
     ocntr = expandByWeekDay(st, fullDays=True)
-    tbl = MarkdownTable(["Day", "Total", "Average"], includeNo=False)
+    tbl = MarkdownTable([":Day", "Total", "Average"], includeNo=False)
     for i, day in enumerate(ocntr):
         avg = ocntr[day].split("/")[1]
         avgMU = "" if avg == "0" else avg
@@ -769,7 +802,7 @@ def expandByWeekDay(st, fullDays=False):
     createDatePropInfo = st[st["_createDateProp"]]
     dayCnter = countDaysInPeriod(createDatePropInfo["firstCreateDate"], createDatePropInfo["lastCreateDate"])
     weekdays=["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"] if fullDays else ["Mon", "Tues", "Wed", "Thurs", "Fri", "Sat", "Sun"]
-    cntr = OrderedDict()
+    cntr = {}
     byWeekDay = st["date_time"]["byWeekDay"]
     for wd in sorted(byWeekDay, key=lambda x: int(x)):
         if wd not in byWeekDay:
@@ -808,19 +841,17 @@ def muSeconds(seconds):
                
 def main():
 
-    assert sys.version_info >= (3, 4)
+    assert sys.version_info >= (3, 6)
 
     try:
         stationNo = sys.argv[1]
     except IndexError:
-        raise SystemExit("Usage _EXE_ STATIONNO")
+        raise SystemExit("Usage _EXE_ STATIONNO [DEID]")
 
-    userSiteDir = SITE_DIR_TEMPL.format(stationNo)
-    if not os.path.isdir(userSiteDir):
-        raise SystemExit("Expect User Site to already exist with its basic contents")
+    ensureWebReportLocations(stationNo)
     
     if len(sys.argv) == 3:
-        deidentify = sys.argv[2]
+        deidentify = True
     else:
         deidentify = False
     
