@@ -5,14 +5,15 @@ import sys
 import os
 import re
 import json
-from collections import defaultdict, Counter, OrderedDict
+from collections import defaultdict, Counter
 from datetime import datetime, timedelta
 from dateutil.relativedelta import relativedelta
 
 from fmqlutils.reporter.reportUtils import MarkdownTable, reportPercent, reportAbsAndPercent
 from fmqlutils.cacher.cacherUtils import metaOfVistA, TMPWORKING_LOCN_TEMPL, FMQLReplyStore, FilteredResultIterator, DATA_LOCN_TEMPL 
-from fmqlutils.schema.reduceTypeUtils import checkDataPresent, muBVC, muBVCOfSTProp
-from fmqlreports.webReportUtils import TOP_MD_TEMPL, SITE_DIR_TEMPL, keyStats, roundFloat, reduce44, filteredResultIterator
+from fmqlutils.typer.reduceTypeUtils import checkDataPresent, muBVC, muBVCOfSTProp
+
+from ..webReportUtils import TOP_MD_TEMPL, SITE_DIR_TEMPL, ensureWebReportLocations, keyStats, roundFloat, reduce44, filteredResultIterator, flattenPropValues
         
 """  
 Encounter (1 or more visits) matched to Appointment => Scheduled Encounter; Appointment matched to Encounter. [1] How well does the system do it? [2] And what's the institution and stop code breakdown. 
@@ -21,6 +22,12 @@ TODO/Evolve: [1] break down between two roles - [a] matching by system (how good
 data reporting (** there are encounters from cancelled and no show appts - can you tell from the visit?; are some non scheduled encounters really just same day extensions of scheduled encounters?**) [2] move to simpler indexes (apptN to Enc etc) for use in reportScheduling
 and in a rewrite of the below to use typer. [3] for each work type broadly (DENTAL etc), do narrow report
 
+TODO: Weekdays only n Nature of Clinics
+- ex/ YAKIMA CBOC [687] - 687HA in FOIA - M-F (https://www.va.gov/directory/guide/facility.asp?id=5319)  
+  < The clinic provides primary medical and mental health care with full-time staff.  The clinic contracts locally in the community and surrounding area for emergent laboratory, x-ray and pharmacy services.
+- ex/ Coeur d' Alene
+(https://www.va.gov/directory/guide/facility.asp?id=5674)
+
 ISSUE/BUG: 541 - too big in first reduction so removed _06 pass. Could break this pass into four.
 
 NICETY TODO: PIE chart the Instit table ie/ those columns into obvious visual break outs
@@ -477,7 +484,7 @@ def webReportEncounters(stationNo, vistaName, indexedDayTimeLocationPatient, cut
                 unmatchedApptsMU = ""
             
             row = [
-                "__[{}](#{})__".format(instit, re.sub(r'[ \(\)]', '_', instit.lower())),
+                "__[{}](#{})__".format(instit, re.sub(r'[ \(\)\.]', '_', instit.lower())),
                 reportAbsAndPercent(institSumm["total"], cross["total"])
             ]
             row.extend([
@@ -495,7 +502,7 @@ def webReportEncounters(stationNo, vistaName, indexedDayTimeLocationPatient, cut
                            
     def muByInstitByDSSnCreateOption(index, instit, institSumm, dssSummsOfInstit, createOptionSummsOfInstit, printOnly=True):  
         
-        imu = "## {}. {} {{#{}}}\n\n".format(index, instit, re.sub(r'[ \(\)]', '_', instit.lower()))
+        imu = "## {}. {} {{#{}}}\n\n".format(index, instit, re.sub(r'[ \(\)\.]', '_', instit.lower()))
         
         # TODO: encounters with cancelled appointments are 'strange', the other
         # side of 'active' appointments with no encounters. These "mis-alignments"
@@ -667,7 +674,7 @@ A copy of VistA __{} [{}]__ was cut on _{}_. For its final 90 full days, startin
         mu += muByInstitByDSSnCreateOption(i, instit, summByInstit[instit], dssSummsByInstit[instit], createOptionSummsByInstit[instit], printOnly=printOnly)
                 
     userSiteDir = SITE_DIR_TEMPL.format(stationNo)
-    printf("Serializing Report to {}".format(userSiteDir))
+    print("Serializing Report to {}".format(userSiteDir))
     if not os.path.isdir(userSiteDir):
         raise Exception("Expect User Site to already exist with its basic contents")
     title = "Encounters {}".format(stationNo)
@@ -707,7 +714,7 @@ def gatherEncounterDataByDay(stationNo, onAndAfterDay, upToDay):
     try:
         gatheredByDay = json.load(open(jsnFl))
     except:
-        printf("Appt source data not yet gathered by day - organizing relevant 44_003, 409_84, 9000010_06 and 9000010 ...")
+        print("Appt source data not yet gathered by day - organizing relevant 44_003, 409_84, 9000010_06 and 9000010 ...")
     else:
         return gatheredByDay
 
@@ -735,12 +742,12 @@ def gatherEncounterDataByDay(stationNo, onAndAfterDay, upToDay):
         try:
             visitsByDay = json.load(open(jsnFl))
         except:
-            printf("9000010 not yet gathered by day for {} to {} ...".format(onAndAfterDay, upToDay))
+            print("9000010 not yet gathered by day for {} to {} ...".format(onAndAfterDay, upToDay))
         else:
             return visitsByDay
 
         visitStartDate = (datetime.strptime(onAndAfterDay, "%Y-%m-%d") - relativedelta(years=1)).strftime("%Y-%m-%d") # one year back
-        printf("Organizing 9000010 starting at {}, one year back from onAndAfter {} ...".format(visitStartDate, onAndAfterDay))
+        print("Organizing 9000010 starting at {}, one year back from onAndAfter {} ...".format(visitStartDate, onAndAfterDay))
         visitsByDay = defaultdict(list)
         visitsByIEN = {} # for provider and parent linking
         parentIENByChildIEN = {} # for checking below
@@ -758,15 +765,15 @@ def gatherEncounterDataByDay(stationNo, onAndAfterDay, upToDay):
         startAtReply = store.firstReplyFileOnOrAfterCreateDay("9000010", startProp, visitStartDate) 
         if startAtReply == "":
             raise Exception("Can't find 9000010 start reply")
-        resourceIter = FilteredResultIterator(dataLocn, "9000010", isOrdered=False, startAtReply=startAtReply)
-        printf("... starting at reply {}".format(startAtReply))
+        resourceIter = FilteredResultIterator(dataLocn, "9000010", startAtReply=startAtReply)
+        print("... starting at reply {}".format(startAtReply))
         for i, resource in enumerate(resourceIter, 1):
             if i == 1:
                 print("\tFirst resource seen {}".format(
                     resource[startProp]["value"] if startProp in resource else "?"
                 ))
             if i % 50000 == 0:
-                printf("\tAnother 50K to {:,}, {:,} taken, {:,} credit stop visit's skipped, last resource day {} - 50K resource create date {}".format(i, cntEncountersInDateRange, cntQualCreditStopVisitsSkipped, resourceDay, resource[startProp]["value"] if startProp in resource else "?"))
+                print("\tAnother 50K to {:,}, {:,} taken, {:,} credit stop visit's skipped, last resource day {} - 50K resource create date {}".format(i, cntEncountersInDateRange, cntQualCreditStopVisitsSkipped, resourceDay, resource[startProp]["value"] if startProp in resource else "?"))
             # Keeps it in the window of onAndAfterDay and upToDay
             resourceDay = getResourceDay(resource, createProp, onAndAfterDay, upToDay)
             if resourceDay == "":
@@ -777,7 +784,7 @@ def gatherEncounterDataByDay(stationNo, onAndAfterDay, upToDay):
                 continue
             cntEncountersInDateRange += 1
             resource["_ien"] = resource["_id"].split("-")[1]
-            for prop in resource:
+            for prop in list(resource):
                 if prop in ["_id", "type", "label"]:
                     del resource[prop]
                     continue
@@ -800,7 +807,7 @@ def gatherEncounterDataByDay(stationNo, onAndAfterDay, upToDay):
             cntTopLevelEncounters += 1
             visitsByDay[resourceDay].append(resource)
             
-        printf("... finished taking {:,} encounters in date range ({}-{}), {:,} are top level, {:,} children, {:,} in range child visits unattached and dropped".format(
+        print("... finished taking {:,} encounters in date range ({}-{}), {:,} are top level, {:,} children, {:,} in range child visits unattached and dropped".format(
             cntEncountersInDateRange, 
             visitStartDate,
             upToDay,
@@ -822,7 +829,7 @@ def gatherEncounterDataByDay(stationNo, onAndAfterDay, upToDay):
 
     def gatherVProviderByVisitIEN(stationNo, onAndAfterDay, upToDay, topVisitIENs, childVisitIENs):
     
-        printf("Organizing 9000010_06 under visits starting at {}, the on and after date ...".format(onAndAfterDay))
+        print("Organizing 9000010_06 under visits starting at {}, the on and after date ...".format(onAndAfterDay))
 
     """
         
@@ -832,17 +839,17 @@ def gatherEncounterDataByDay(stationNo, onAndAfterDay, upToDay):
         try:
             oldApptsByDay = json.load(open(jsnFl))
         except:
-            printf("44_003 not yet gathered by day for {} to {} ...".format(onAndAfterDay, upToDay))
+            print("44_003 not yet gathered by day for {} to {} ...".format(onAndAfterDay, upToDay))
         else:
             return oldApptsByDay
 
-        printf("Organizing 44_003 (walking all) ...")
+        print("Organizing 44_003 (walking all) ...")
         oldApptsByDay = defaultdict(list)
         createProp = "appointment_date_time"
         taken = 0
         for i, resource in enumerate(filteredResultIterator(stationNo, "44_003", useRF=True), 1):
             if i % 50000 == 0:
-                printf("\tAnother 50K to {:,}, {:,} taken".format(i, taken))
+                print("\tAnother 50K to {:,}, {:,} taken".format(i, taken))
             resourceDay = getResourceDay(resource, createProp, onAndAfterDay, upToDay)
             # filter to window
             if resourceDay == "":
@@ -854,7 +861,7 @@ def gatherEncounterDataByDay(stationNo, onAndAfterDay, upToDay):
                     del resource[prop]
             flattenPropValues(resource)
             oldApptsByDay[resourceDay].append(resource)
-        printf("... finished processing {:,} 44_003's, {:,} taken".format(i, taken))
+        print("... finished processing {:,} 44_003's, {:,} taken".format(i, taken))
 
         json.dump(oldApptsByDay, open(jsnFl, "w"))
     
@@ -868,17 +875,17 @@ def gatherEncounterDataByDay(stationNo, onAndAfterDay, upToDay):
         try:
             newApptsByDay = json.load(open(jsnFl))
         except:
-            printf("409_84 not yet gathered by day for {} to {} ...".format(onAndAfterDay, upToDay))
+            print("409_84 not yet gathered by day for {} to {} ...".format(onAndAfterDay, upToDay))
         else:
             return newApptsByDay
         
-        printf("Organizing 409_84 (walking all, filtering to window) ...")
+        print("Organizing 409_84 (walking all, filtering to window) ...")
         newApptsByDay = defaultdict(list)
         createProp = "starttime"
         taken = 0
         for i, resource in enumerate(filteredResultIterator(stationNo, "409_84"), 1):
             if i % 50000 == 0:
-                printf("\tAnother 50K to {:,}, {:,} taken".format(i, taken))
+                print("\tAnother 50K to {:,}, {:,} taken".format(i, taken))
             resourceDay = getResourceDay(resource, createProp, onAndAfterDay, upToDay)
             # filter to window
             if resourceDay == "":
@@ -890,7 +897,7 @@ def gatherEncounterDataByDay(stationNo, onAndAfterDay, upToDay):
                     del resource[prop]
             flattenPropValues(resource) 
             newApptsByDay[resourceDay].append(resource)
-        printf("... finished processing {:,} 409_84, {:,} taken".format(i, taken))
+        print("... finished processing {:,} 409_84, {:,} taken".format(i, taken))
         
         json.dump(newApptsByDay, open(jsnFl, "w"))
 
@@ -898,7 +905,7 @@ def gatherEncounterDataByDay(stationNo, onAndAfterDay, upToDay):
 
     newApptsByDay = gatherNewApptsByDay(stationNo, onAndAfterDay, upToDay)
                     
-    printf("Returning Organized Data")     
+    print("Returning Organized Data")     
     return visitsByDay, oldApptsByDay, newApptsByDay
         
 """
@@ -1072,7 +1079,7 @@ def categorizeVisitAppts(info, noneApptOnlyDetails=None, matchedDetails=None, ma
             cnt = 0
             for i, dt in enumerate(sorted(dts), 1):
                 try:
-                    dtDT = datetime.strptime(dt, "%Y-%m-%dT%H:%M:%SZ")
+                    dtDT = datetime.strptime(dt, "%Y-%m-%dT%H:%M:%S")
                 except:
                     print("** PROBLEM BAD DATE TIME: {}".format(dt))
                     continue
@@ -1513,33 +1520,6 @@ def categorizeVisitAppts(info, noneApptOnlyDetails=None, matchedDetails=None, ma
         return matchScheduledVisit(info, apptType, apptN, apptL)
     return matchPCEVisit(info, apptType, apptN, apptL)
 
-def printf(x, flush=True): # same as print in Python 3
-    print(x)
-    sys.stdout.flush()
-
-"""
-Utility for making tmp's - flatten out dates, pters etc
-
-TODO: add to reduction utilities. 
-"""
-def flattenPropValues(resource, nixTypeLabel=True): # not recursive
-    if nixTypeLabel:
-        for prop in ["type", "label"]:
-            if prop in resource:
-                del resource[prop]
-    for prop in resource: 
-        if isinstance(resource[prop], list):
-            for mresource in resource[prop]:
-                flattenPropValues(mresource, False)
-            continue
-        if not isinstance(resource[prop], dict):
-            continue
-        if "value" in resource[prop]:
-            resource[prop] = resource[prop]["value"]
-            continue
-        if "id" in resource[prop]:
-            resource[prop] = "{} [{}]".format(resource[prop]["label"], resource[prop]["id"]) 
-            
 # ###################### Per 9000010 Enforce #####################
 
 """
@@ -1717,16 +1697,14 @@ def enforce9000010Quad(stationNo):
 
 def main():
 
-    assert sys.version_info >= (3, 4)
+    assert sys.version_info >= (3, 6)
 
     try:
         stationNo = sys.argv[1]
     except IndexError:
         raise SystemExit("Usage _EXE_ STATIONNO")
 
-    userSiteDir = SITE_DIR_TEMPL.format(stationNo)
-    if not os.path.isdir(userSiteDir):
-        raise SystemExit("Expect User Site to already exist with its basic contents")
+    ensureWebReportLocations(stationNo)
 
     meta = metaOfVistA(stationNo)
     vistaName = "VistA" if "name" not in meta else meta["name"]
