@@ -5,14 +5,15 @@ import sys
 import os
 import re
 import json
-from collections import defaultdict, Counter, OrderedDict
+from collections import defaultdict, Counter
 from datetime import datetime
 import numpy
 
 from fmqlutils.reporter.reportUtils import MarkdownTable, reportPercent, reportAbsAndPercent
 from fmqlutils.cacher.cacherUtils import metaOfVistA
 from fmqlutils.typer.reduceTypeUtils import splitTypeDatas, combineSubTypes, checkDataPresent, singleValue, muBVC, muBVCOfSTProp
-from fmqlreports.webReportUtils import filteredResultIterator, reduce200, reduce4, reduce44, TOP_MD_TEMPL, SITE_DIR_TEMPL, keyStats, roundFloat
+
+from ..webReportUtils import filteredResultIterator, reduce200, reduce4, reduce44, TOP_MD_TEMPL, SITE_DIR_TEMPL, ensureWebReportLocations, keyStats, roundFloat
 
 """
 Overall: Scheduling w/VSE focus
@@ -21,6 +22,8 @@ TODO: [1] match webReportEncounter's nuance of noshow(VSE only)/cancelled/active
 for appts and carry it through into tables ie/ can index triple ST by workload and instit 
 in detailed report below. [2] prelude to showing % of each type with matching enc - mainly/only active with some active's still being left as orphans. [3] Also can show legacy to VSE matchups where not all legacy has a VSE and vica versa including workload
 type.
++
+by institution and its stationNo -- seems to take that of first st by institution, st broken by Hospital Location. If locn is wrongly allocated then will end up with wrong station no for Instit in table?
 
 per institution (has station number), showing scheduling based on stop codes. Additional nuance provided by normalized physical location names and, if many locations, forms/templates of locations. Handles case where only legacy (effectively) is working
 ie/ VSE little or none for recent 3 month period (ex/ Columbus)
@@ -216,7 +219,7 @@ def reportAppointmentsLegacy44(stationNo, vistaName, cutDate):
     firstSignificantDate = re.sub(r'\-', '/', sigSortedADT[0])
     lastSignificantDate = re.sub(r'\-', '/', sigSortedADT[-1])
 
-    allDAY90, stsDAY90 = splitTypeDatas(stationNo, "44_003", reductionLabel="DAY90", expectSubTypeProperties=["appointment_container_", "#appointment_cancelled"])     
+    allDAY90, stsDAY90 = splitTypeDatas(stationNo, "44_003", reductionLabel="DAY90", expectSubTypeProperties=["appointment_container_", "#appointment_cancelled"], expectNotEfficientWalked=True)     
     stsDAY90Active = [st for st in stsDAY90 if re.search(r'ABSENT', st["_subTypeId"])]
     # TODO: eventually will only do non cancelled as cuts out ZZ's too
     stsDAY90Cancelled = [st for st in stsDAY90 if re.search(r'PRESENT', st["_subTypeId"])]
@@ -284,7 +287,7 @@ For the 90 days before this VistA copy was cut (_{}_), the System shows <span cl
         raise Exception("Expect User Site to already exist with its basic contents")
     title = "Legacy Appointments {}".format(stationNo)
     mu = TOP_MD_TEMPL.format(title, "Legacy Appointments") + mu
-    open(userSiteDir + "appointmentsLegacy44.md", "w").write(mu)
+    open(userSiteDir + "appointmentsLegacy.md", "w").write(mu)
 
 # ################################# Common MU's/ HL Enhance ###################
 
@@ -700,16 +703,14 @@ def reduce409_831(stationNo):
 
 def main():
 
-    assert sys.version_info >= (3, 4)
+    assert sys.version_info >= (3, 6)
 
     try:
         stationNo = sys.argv[1]
     except IndexError:
         raise SystemExit("Usage _EXE_ STATIONNO")
 
-    userSiteDir = SITE_DIR_TEMPL.format(stationNo)
-    if not os.path.isdir(userSiteDir):
-        raise SystemExit("Expect User Site to already exist with its basic contents")
+    ensureWebReportLocations(stationNo)
     
     meta = metaOfVistA(stationNo)
     vistaName = "VistA" if "name" not in meta else meta["name"]
