@@ -251,11 +251,10 @@ def reduce200(stationNo, onlyUsers=None, forceNew=False, coreOnly=False):
         info = {"hasProps": sorted([key for key in resource if key not in set(["_id", "label", "type"])])}
         
         if "creator" in resource:
-            if resource["creator"]["id"] == "200-0":
-                info["is_created_by_0"] = True
-            else:
-                createdId = "{} [{}]".format(resource["creator"]["label"], resource["creator"]["id"])
-                info["creator"] = createdId
+            # Only v1.1 will even have this ie/ not V10 on
+            if resource["creator"]["id"] != "200-0":
+                creatorRef = "{} [{}]".format(resource["creator"]["label"], resource["creator"]["id"])
+                info["creator"] = creatorRef
         
         if "date_entered" in resource:
             info["date_entered"] = resource["date_entered"]["value"]
@@ -270,7 +269,7 @@ def reduce200(stationNo, onlyUsers=None, forceNew=False, coreOnly=False):
         if "secid" in resource:
             info["secid"] = resource["secid"]
             if "subject_organization" in resource and resource["subject_organization"] != "Department Of Veterans Affairs":
-                raise Exception("Expected Subject Org for sec id to always be VA!")
+                print("** WARNING: Expected Subject Org for sec id to always be VA!")
             
         if "termination_date" in resource:
             info["termination_date"] = resource["termination_date"]["value"]
@@ -302,6 +301,8 @@ def reduce200(stationNo, onlyUsers=None, forceNew=False, coreOnly=False):
         if "secondary_menu_options" in resource:
             smos = set() # allows for dups with crude SMO addition
             for smo in resource["secondary_menu_options"]:
+                if "secondary_menu_options" not in smo:
+                    continue
                 if not re.match(r'\d+$', smo["secondary_menu_options"]["label"]):
                     smos.add(smo["secondary_menu_options"]["label"])
             if len(smos):
@@ -318,6 +319,8 @@ def reduce200(stationNo, onlyUsers=None, forceNew=False, coreOnly=False):
         if "keys" in resource:
             keys = set() 
             for key in resource["keys"]:
+                if "key" not in key:
+                    continue
                 if not re.match(r'\d+$', key["key"]["label"]):
                     keys.add(key["key"]["label"])
             if len(keys):
@@ -538,6 +541,19 @@ def reduce44(stationNo, useRF=True): # do from RF
     json.dump(infosByIEN, open(jsnFl, "w"), indent=4)
     return infosByIEN
 
+"""
+name, short_name | names
+status, inactive_facility_flag | National or Local and if active. Nationals are maintained by MFS and should have a station number (expect Nationals same across VistAs)
+station_number, official_va_name | Nationals should have these and they should not be changed by a site (expect same cross VistAs)
+facility_type | facility_type (pter to meta file) replaces va_type_code
+reporting_station | "pointer back to the Institution file that indicates the site that does the reporting to VACO." TODO: if SPOK for clinics etc
+location_timezone | time zone of location
+multi-division facility | TODO: see if useful
+
+For now ignoring, address, agency_code etc
+
+TODO: push into reduction useful for nationals -- use association
+"""
 def reduce4(stationNo):
     jsnFl = TMPWORKING_LOCN_TEMPL.format(stationNo) + "4Reduction.json"
     try:
@@ -551,14 +567,65 @@ def reduce4(stationNo):
         ien = resource["_id"].split("-")[1]
         red = {"ien": ien, "label": resource["name"]}
         infosByIEN[ien] = red
-        if "reporting_station" in resource:
-            red["reporting_station"] = "{} [{}]".format(resource["reporting_station"]["label"], resource["reporting_station"]["id"].split("-")[1]) 
-        for prop in ["va_type_code", "status", "npi", "official_va_name", "station_number"]:
+        # OPEN QUESTION: any useful or now used for national ... see
+        for refProp in ["reporting_station", "facility_type", "location_timezone"]:
+            if refProp not in resource:
+                continue
+            red[refProp] = "{} [{}]".format(resource[refProp]["label"], resource[refProp]["id"].split("-")[1])
+        if "inactive_facility_flag" in resource:
+            red["isInactive"] = True
+        if "status" in resource:
+            if resource["status"] == "N:National":
+                red["isNational"] = True
+        for prop in ["short_name", "multi_division_facility", "npi", "official_va_name", "station_number"]:
             if prop not in resource:
                 continue
             red[prop] = resource[prop]
+        # VISN, Parent Facility etc
+        if "associations" in resource:
+            for association in resource["associations"]:
+                atyp = association["associations"]
+                apred = re.sub(r' ', '_', atyp["label"].lower())
+                ref = association["parent_of_association"]
+                if ref["id"] == resource["_id"]:
+                    continue # avoid self reference in Puget being parent of Puget
+                red[apred] = ref
+        # EX/ VISTANUM == stationNumber but npi too (also explicit?)
+        # ... has status and effective_date_time too
+        # ... https://npiregistry.cms.hhs.gov/registry/provider-view/1477502946 ... 
+        # America Lake
+        if "identifier" in resource:
+            for identifier in resource["identifier"]:
+                ityp = identifier["coding_system"]
+                ipred = re.sub(r' ', '_', ityp["label"].lower()) + "_id"
+                id_02 = identifier["id_02"]
+                red[ipred] = id_02
+        # taxonomy_code - reduce to list
     json.dump(infosByIEN, open(jsnFl, "w"), indent=4)
     return infosByIEN
+
+"""
+Utility for making tmp's - flatten out dates, pters etc
+
+TODO: add to reduction utilities.
+"""
+def flattenPropValues(resource, nixTypeLabel=True): # not recursive
+    if nixTypeLabel:
+        for prop in ["type", "label"]:
+            if prop in resource:
+                del resource[prop]
+    for prop in resource.keys():
+        if isinstance(resource[prop], list):
+            for mresource in resource[prop]:
+                flattenPropValues(mresource, False)
+            continue
+        if not isinstance(resource[prop], dict):
+            continue
+        if "value" in resource[prop]:
+            resource[prop] = resource[prop]["value"]
+            continue
+        if "id" in resource[prop]:
+            resource[prop] = "{} [{}]".format(resource[prop]["label"], resource[prop]["id"])
     
 # ############################ Simple Reusable MD Makers ###############################
 
